/******************************************************************************
 *
 *   This file is part of LC1 - Load Control application 1, by Sven Ruin.
 *
 *   Copyright (C) 2010-2020  TEROC AB.
 *
 *   LC1 is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   LC1 is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with LC1.  If not, see <http://www.gnu.org/licenses/>.
 *
 *   Note that LC1 depends on other software, with different licensing
 *   conditions.
 *
 *   We are thankful to Echelon/Adesto/Dialog and others, who have
 *   provided helpful examples and support.
 *
 *   For more information, contact Sven Ruin, sven.ruin@teroc.se
 *
 *   See version number below in init section.
 *   
 *   Revision history:
 *   Ver 0.5 (2019-10-29) GNU General Public License, etc
 *   Ver 0.6 (2020-11-25) Added algorithm 2 and updated XIF
 *
 ******************************************************************************/

#include "UFPTLC1.h"
// => section dependent includes
// <= section dependent includes
using namespace _0000000000000000_0_;

using namespace SmartServer::FPM_LIB_VERSION;
using namespace _9FFD030000001000_5__UFPTLC1_APP;

//-------------------------------------------------------------------
// => section datapoint variable declarations. DO NOT REMOVE THIS SECTION'S COMMENT!
DECLARE( _0000000000000000_0_::SNVT_amp, nviInvIbat, INPUT_DP )
DECLARE( _0000000000000000_0_::SNVT_switch, nviInvCommOK, INPUT_DP )
DECLARE( _0000000000000000_0_::SNVT_amp, nviInvIin, INPUT_DP )
DECLARE( _0000000000000000_0_::SNVT_amp, nviInvIout, INPUT_DP )
DECLARE( _0000000000000000_0_::SNVT_count, nviInvMode, INPUT_DP )
DECLARE( _0000000000000000_0_::SNVT_count, nviInvBatCyPh, INPUT_DP )
DECLARE( _0000000000000000_0_::SNVT_temp, nviInvTbat, INPUT_DP )
DECLARE( _0000000000000000_0_::SNVT_switch, nviInvTransf, INPUT_DP )
DECLARE( _0000000000000000_0_::SNVT_volt, nviInvUbat, INPUT_DP )
DECLARE( _0000000000000000_0_::SNVT_volt, nviInvUin, INPUT_DP )
DECLARE( _0000000000000000_0_::SNVT_count, nviOpMode, INPUT_DP )
DECLARE( _0000000000000000_0_::SNVT_switch, nviDefLevel, INPUT_DP )
DECLARE( _0000000000000000_0_::SNVT_amp, nviMaxInvIin, INPUT_DP )
DECLARE( _0000000000000000_0_::SNVT_amp, nviMaxInvIout, INPUT_DP )
DECLARE( _0000000000000000_0_::SNVT_count, nviPolicy, INPUT_DP )
DECLARE( _0000000000000000_0_::SNVT_amp, nviMinInvIin, INPUT_DP )
DECLARE( _0000000000000000_0_::SNVT_temp, nviMinInvTbat, INPUT_DP )
DECLARE( _0000000000000000_0_::SNVT_volt, nviMinInvUbat, INPUT_DP )
DECLARE( _0000000000000000_0_::SNVT_volt, nviMinInvUin, INPUT_DP )
DECLARE( _0000000000000000_0_::SNVT_count, nvoLevel, OUTPUT_DP )
DECLARE( _0000000000000000_0_::SNVT_switch, nvoLoadVal1, OUTPUT_DP )
DECLARE( _0000000000000000_0_::SNVT_switch, nvoLoadVal2, OUTPUT_DP )
DECLARE( _0000000000000000_0_::SNVT_switch, nvoLoadVal3, OUTPUT_DP )
DECLARE( _0000000000000000_0_::SNVT_switch, nvoLoadVal4, OUTPUT_DP )
DECLARE( _0000000000000000_0_::SNVT_switch, nviLoadValFb1, INPUT_DP )
DECLARE( _0000000000000000_0_::SNVT_switch, nviLoadValFb2, INPUT_DP )
DECLARE( _0000000000000000_0_::SNVT_switch, nviLoadValFb3, INPUT_DP )
DECLARE( _0000000000000000_0_::SNVT_switch, nviLoadValFb4, INPUT_DP )
DECLARE( _0000000000000000_0_::SNVT_amp_f, nviLoadCurr1, INPUT_DP )
DECLARE( _0000000000000000_0_::SNVT_amp_f, nviLoadCurr2, INPUT_DP )
DECLARE( _0000000000000000_0_::SNVT_amp_f, nviLoadCurr3, INPUT_DP )
DECLARE( _0000000000000000_0_::SNVT_amp_f, nviLoadCurr4, INPUT_DP )
DECLARE( _0000000000000000_0_::SNVT_amp_f, nviMaxCurr1, INPUT_DP )
DECLARE( _0000000000000000_0_::SNVT_amp_f, nviMaxCurr2, INPUT_DP )
DECLARE( _0000000000000000_0_::SNVT_amp_f, nviMaxCurr3, INPUT_DP )
DECLARE( _0000000000000000_0_::SNVT_amp_f, nviMaxCurr4, INPUT_DP )
// <= section datapoint variable declarations. DO NOT REMOVE THIS SECTION'S COMMENT!
//-------------------------------------------------------------------
// If variables are needed that accommodate values local to each fpm instance 
// these variables are declared here:
// DECLARE_FB_INSTANCE_LOCAL( int, _nInstanceNo )
 
// Sven's code - begin
DECLARE_FB_INSTANCE_LOCAL( int, step )
DECLARE_FB_INSTANCE_LOCAL( int, repeat )
DECLARE_FB_INSTANCE_LOCAL( int, margin )
// Sven's code - end

//-------------------------------------------------------------------
// ==> the one and only instance
//-------------------------------------------------------------------
static FPM::TStarter<CUFPTLC1> STARTER( FPM_MODULE_NAME );

//-------------------------------------------------------------------
// ==> extern "C" commands (typically not needed)
//-------------------------------------------------------------------
#ifdef __cplusplus
  extern "C" STATUS fnStartFPM_UFPTLC1() {
    return STARTER.StartModule();
  }
  
  //-------------------------------------------------------------------
  extern "C" STATUS fnStopFPM_UFPTLC1() { 
    return STARTER.StopModule();
  }
  
  //-------------------------------------------------------------------
  extern "C" void fnFPMLibVersion() { 
    STARTER.GetFPMLibVersion();
  }
#endif


//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// class: CUFPTLC1 
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
CUFPTLC1 *CUFPTLC1::m_pSingleton = NULL;

// -------------------------------------------------------------------
// Class: CUFPTLC1  [CStarter METHODs]
// -------------------------------------------------------------------
STATUS CUFPTLC1::StartModule(const char* pszModuleName)
{
  string      sInitString;
  E_ErrorCode eFaultCode = eFENone;
  CUFPTLC1::GetSingleton()->InitApp( sInitString, eFaultCode );
  if (eFENone == eFaultCode) {
    return OK;
  }
  printf ( "Error while starting FPM UFPTLC1: %s\n", sInitString.c_str() );

  return ERROR;
}

STATUS CUFPTLC1::StopModule(const char* pszModuleName)
{
  if (ERROR == CUFPTLC1::GetSingleton()->ShutdownBase()) {
    return ERROR;
  }
  CUFPTLC1::DestroySingleton();

  return OK;
}

// -------------------------------------------------------------------
// CUFPTLC1   [CONSTRUCTOR / DESTRUCTOR]
// -------------------------------------------------------------------
CUFPTLC1::CUFPTLC1()
  : CFPM_App( FPM_MODULE_NAME, CFPM_App::eApplication )

// Sven's code - begin
, m_oTimer1( this )  // initialize timer
// Sven's code - end
{
}

CUFPTLC1::~CUFPTLC1()
{
}

// This routine is called once, while starting up the module
void CUFPTLC1::Initialize()
{
  // Sven's code - begin
  SNVT_switch tmp;
  tmp.state = 0;
  tmp.value = 0;
  step = 0;
  repeat = 0;
  margin = 0;
  nviMaxInvIin = 130;  // 13 A
  nviMaxInvIout = 130;  // 13 A
  nviMinInvIin = 40;  // 4 A
  nviMinInvUbat = 470;  // 47 V
  nviPolicy = 20;       // hex: 14 (algorithm = 1, loads = 4)
  nvoLevel = 0;
  nvoLoadVal1 = tmp;
  nvoLoadVal2 = tmp;
  nvoLoadVal3 = tmp;
  nvoLoadVal4 = tmp;
  m_oTimer1.Start( FPM_TF_REPEAT, 2000 );  // start timer

  printf("Init Load Control application 1 ver 0.6 \n");
  // Sven's code - end
}

void CUFPTLC1::Shutdown()
{
  // Sven's code - begin
  StopAllTimers();
  // Sven's code - end
}

void CUFPTLC1::Work()
{
}

void CUFPTLC1::OnTimer()
{
  // Sven's code - begin
  const int LEVELSMASK = 15;  // binary: 00001111
  const int ALGORMASK = 240;  // binary: 11110000
  int n;
  SNVT_switch tmp;
  n = 0;
  if (m_oTimer1.Expired())
  {
  	if (step < 150)
  	  step = step + 1;
  	if (step >= 75)
  	{
  	  switch(nviPolicy & ALGORMASK)
  	  {
  	    case 16:  // algorithm = 1 (loads on when inverter AC output current not too high and battery voltage high)
     	  if (((nviInvIout + margin) < *nviMaxInvIout) && (nviInvUbat > *nviMinInvUbat) && (nvoLevel < (nviPolicy & LEVELSMASK)))
    	  {
    	    nvoLevel = nvoLevel + 1;
    	    n = nvoLevel;
    	    tmp.state = 1;
    	    tmp.value = 200;
    	  }
    	  else if (((nviInvIout > *nviMaxInvIout) || (nviInvUbat < *nviMinInvUbat) ) && (step >= 150) && (nvoLevel > 0))
    	  {
    	    n = nvoLevel;
    	    tmp.state = 0;
    	    tmp.value = 0;
    	    nvoLevel = nvoLevel - 1;
    	    margin = 1000;
    	  }
  	    case 32:  // algorithm = 2 (loads on when inverter AC input current low during charger mode)
     	  if (((nviInvIin + margin) < *nviMinInvIin) && (nviInvMode == 2) && (nvoLevel < (nviPolicy & LEVELSMASK)))
    	  {
    	    nvoLevel = nvoLevel + 1;
    	    n = nvoLevel;
    	    tmp.state = 1;
    	    tmp.value = 200;
    	  }
    	  else if (((nviInvIin > *nviMaxInvIin) || (nviInvMode != 2) ) && (step >= 150) && (nvoLevel > 0))
    	  {
    	    n = nvoLevel;
    	    tmp.state = 0;
    	    tmp.value = 0;
    	    nvoLevel = nvoLevel - 1;
    	    margin = 1000;
    	  }
  	  }
  	  if (n > 0)
  	  {
  	    step = 0;
  	    repeat = 0;
  	    printf("Set%i\n", n);
  	    switch(n)
  	    {
  	      case 1:
  	        nvoLoadVal1 = tmp;
  	        break;
  	      case 2:
  	        nvoLoadVal2 = tmp;
  	        break;
  	      case 3:
  	        nvoLoadVal3 = tmp;
  	        break;
  	      case 4:
  	        nvoLoadVal4 = tmp;
  	        break;
  	    }
  	  }
  	}
  	switch(repeat)
  	{
  	  case 1:
  	    PROPAGATE(nvoLoadVal1);
  	    printf("Repeat1\n");
  	    break;
  	  case 2:
  	    PROPAGATE(nvoLoadVal2);
  	    printf("Repeat2\n");
  	    break;
  	  case 3:
  	    PROPAGATE(nvoLoadVal3);
  	    printf("Repeat3\n");
  	    break;
  	  case 4:
  	    PROPAGATE(nvoLoadVal4);
  	    printf("Repeat4\n");
  	    break;
    }
  	if (step >= 6)
      if (repeat < 4)
        repeat = repeat + 1;
      else
        repeat = 1;
    else
      repeat = n;
  	if (step >= 150)
      margin = 0;
  }
  // Sven's code - end
}
